package com.salesmanago.am_monitor_plugin;

import android.content.Context;
import android.util.Log;

import com.appmanago.exception.NotFoundRequiredPropertiesException;
import com.appmanago.lib.AmMonitor;
import com.appmanago.lib.AmProperties;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class AmMonitorAdapter {

    public static final String LOG_TAG = "AmMonitorAdapter";
    private static final AmProperties EMPTY_PROPERTIES = new AmProperties();

    private final Map<String, AmMonitor> modulesMonitors = new HashMap<>();
    private final AmMonitor amMonitor;
    private final Context context;


    public AmMonitorAdapter(Context context) {
        this.context = context;

        try {
            amMonitor = AmMonitor.initLibrary(context);
            Log.i(LOG_TAG , "AmMonitor init");
        } catch (NotFoundRequiredPropertiesException e) {
            throw new RuntimeException("Exception while creating native SMM module", e);
        }
    }

    public void create(String moduleSimpleId) {
        try {
            AmMonitor monitor = AmMonitor.initLibrary(context);
            modulesMonitors.put(moduleSimpleId, monitor);
            Log.i(LOG_TAG , "AmMonitor create module");
        } catch (NotFoundRequiredPropertiesException e) {
            Log.e(LOG_TAG, "Exception while creating AmMonitor", e);
        }
    }

    public void destroy(String moduleSimpleId) {
        modulesMonitors.remove(moduleSimpleId);
    }

    public void eventStarted(String moduleSimpleId) {
        AmMonitor monitor = modulesMonitors.get(moduleSimpleId);
        if (monitor == null) {
            Log.e(LOG_TAG, "Attempted to call \"start\" method before \"create\"");
            return;
        }
        monitor.eventStarted(moduleSimpleId, EMPTY_PROPERTIES);

    }

    public void eventEnded(String moduleSimpleId) {
        AmMonitor monitor = modulesMonitors.get(moduleSimpleId);
        if (monitor == null) {
            Log.e(LOG_TAG, "Attempted to call \"end\" method before \"create\"");
            return;
        }
        monitor.eventEnded(moduleSimpleId, EMPTY_PROPERTIES);

    }

    public void clicked(String moduleSimpleId, String functionSimpleId) {
        AmMonitor monitor = modulesMonitors.get(moduleSimpleId);
        if (monitor == null) {
            Log.e(LOG_TAG, "Attempted to call \"clicked\" method before module \"create\"");
            return;
        }
        monitor.clicked(functionSimpleId, EMPTY_PROPERTIES);
    }

    public void eventCustom(String customEventName, Map<String, Object> properties) {
        AmProperties amProperties = convertToAmProperties(properties);
        amMonitor.eventCustom(customEventName, amProperties);
    }

    public void syncEmail(String email) {
        amMonitor.syncEmail(email);
    }

    public void syncMsisdn(String msisdn) {
        amMonitor.syncMsisdn(msisdn);
    }

    public void sendUserProperties(Map<String, Object> properties) {
        AmProperties amProperties = convertToAmProperties(properties);
        amMonitor.sendUserProperties(amProperties);
    }

    public void sendLocation(String latitude, String longitude) {
        amMonitor.sendLocation(latitude, longitude);
    }

    private AmProperties convertToAmProperties(Map<String, Object> properties) {
        AmProperties amProperties = new AmProperties();

        for (String obj : properties.keySet()) {
            Map<String, Object> map = (Map<String, Object>)properties.get(obj);
            switch ((String) Objects.requireNonNull(map.get("type"))) {
                case "text":
                    amProperties.addProperty(obj, (String) map.get("value"));
                    break;
                case "integer":
                    amProperties.addProperty(obj, (int) map.get("value"));
                    break;
                case "double":
                    amProperties.addProperty(obj, (double) map.get("value"));
                    break;
                case "date":
                    Date date = new Date((long) map.get("value"));
                    amProperties.addProperty(obj, date);
                    break;
                case "boolean":
                    amProperties.addProperty(obj, (boolean) map.get("value"));
                    break;
                default:
                    break;
            }
        }

        return amProperties;

    }

}
