package com.salesmanago.am_monitor_plugin;

import android.content.Context;

import androidx.annotation.NonNull;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** AmMonitorPlugin */
public class AmMonitorPlugin implements FlutterPlugin, MethodCallHandler {

  private MethodChannel channel;
  private AmMonitorAdapter amMonitorAdapter;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "com.salesmanago/ammonitor");
    channel.setMethodCallHandler(this);

    Context applicationContext = flutterPluginBinding.getApplicationContext();
    amMonitorAdapter = new AmMonitorAdapter(applicationContext);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "create":
        amMonitorAdapter.create(call.argument("moduleSimpleId"));
        break;
      case "destroy":
        amMonitorAdapter.destroy(call.argument("moduleSimpleId"));
        break;
      case "eventStarted":
        amMonitorAdapter.eventStarted(call.argument("moduleSimpleId"));
        break;
      case "eventEnded":
        amMonitorAdapter.eventEnded(call.argument("moduleSimpleId"));
        break;
      case "clicked":
        amMonitorAdapter.clicked(call.argument("moduleSimpleId"), call.argument("functionSimpleId"));
        break;
      case "eventCustom":
        Map<String, Object> properties = call.argument("properties");
        if (properties != null) {
          amMonitorAdapter.eventCustom(call.argument("customEventName"), properties);
        }
        break;
      case "syncEmail":
        amMonitorAdapter.syncEmail(call.argument("email"));
        break;
      case "syncMsisdn":
        amMonitorAdapter.syncMsisdn(call.argument("msisdn"));
        break;
      case "sendUserProperties":
        Map<String, Object> userProperties = call.argument("properties");
        if (userProperties != null) {
          amMonitorAdapter.sendUserProperties(userProperties);
        }
        break;
      case "sendLocation":
        amMonitorAdapter.sendLocation(call.argument("latitude"), call.argument("longitude"));
        break;
      default:
        result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
