#import "AmMonitorPlugin.h"
#import <AMMonitor/AMProperties.h>
#import <AMMonitor/AMMonitor.h>
#import <AMMonitor/AMNotification.h>


@implementation AmMonitorPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"com.salesmanago/ammonitor"
            binaryMessenger:[registrar messenger]];
  AmMonitorPlugin* instance = [[AmMonitorPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
  [registrar addApplicationDelegate:instance];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else if ([@"eventStarted" isEqualToString:call.method]) {
      NSString* simpleId = (NSString*)call.arguments[@"moduleSimpleId"];
      [[AMMonitor sharedInstance] trackEventWithEventType:@"START" andSimpleId:simpleId andProperties:nil];
  } else if ([@"eventEnded" isEqualToString:call.method]) {
      NSString* simpleId = (NSString*)call.arguments[@"moduleSimpleId"];
      [[AMMonitor sharedInstance] trackEventWithEventType:@"END" andSimpleId:simpleId andProperties:nil];
  } else if ([@"clicked" isEqualToString:call.method]) {
      NSString* simpleId = (NSString*)call.arguments[@"functionSimpleId"];
      [[AMMonitor sharedInstance] trackEventWithEventType:@"CLICKED" andSimpleId:simpleId andProperties:nil];
  } else if ([@"eventCustom" isEqualToString:call.method]) {
      NSDictionary* propertiesToConvert = (NSDictionary*)call.arguments[@"properties"];
      AMProperties* properties = convertToAMProperties(propertiesToConvert);
      NSString* eventType = (NSString*)call.arguments[@"customEventName"];
      [[AMMonitor sharedInstance] trackCustomEvent:properties withEventType:eventType];
  } else if ([@"syncEmail" isEqualToString:call.method]) {
      NSString* email = (NSString*)call.arguments[@"email"];
      [[AMMonitor sharedInstance] syncEmail:email];
  } else if ([@"syncMsisdn" isEqualToString:call.method]) {
      NSString* msisdn = (NSString*)call.arguments[@"msisdn"];
      [[AMMonitor sharedInstance] syncMsisdn:msisdn];
  } else if ([@"sendUserProperties" isEqualToString:call.method]) {
      NSDictionary* propertiesToConvert = (NSDictionary*)call.arguments[@"properties"];
      AMProperties* properties = convertToAMProperties(propertiesToConvert);
      [[AMMonitor sharedInstance] setUserProperties:properties];
  } else if ([@"sendLocation" isEqualToString:call.method]) {
      NSString* latitude = (NSString*)call.arguments[@"latitude"];
      NSString* longitude = (NSString*)call.arguments[@"longitude"];
      [[AMMonitor sharedInstance] recordLocationWithLatitude:latitude andLongitude:longitude];
  }
}

static AMProperties* convertToAMProperties(NSDictionary* properties) {
  AMProperties* amProperties = [[AMProperties alloc] init];
  for (NSString* name in properties) {
    NSDictionary* property = properties[name];
    NSString* type = property[@"type"];
    if ([@"text" isEqualToString:type]) {
      NSString* value = property[@"value"];
      [amProperties addString:value withName:name];
    } else if ([@"integer" isEqualToString:type]) {
      NSNumber* value = property[@"value"];
      int intValue = [value intValue];
      [amProperties addNumber:intValue withName:name];
    } else if ([@"double" isEqualToString:type]) {
      NSNumber* value = property[@"value"];
      NSDecimal decimal = [value decimalValue];
      NSDecimalNumber* decimalValue = [NSDecimalNumber decimalNumberWithDecimal:decimal];
      [amProperties addDecimalNumber:decimalValue withName:name];
    } else if ([@"date" isEqualToString:type]) {
      NSString* value = property[@"value"];
      long dateMillis = [value longLongValue];
      [amProperties addDate:dateMillis withName:name];
    } else if ([@"boolean" isEqualToString:type]) {
      bool value = property[@"value"];
      [amProperties addBoolean:value withName:name];
    } else {
      NSLog(@"AMMonitorAdapter: Invalid property type");
    }
}
  return amProperties;
}

void (^dialogHandler)(AMNotification *) = ^(AMNotification *notification) {
       UIAlertAction *okAction = [UIAlertAction actionWithTitle:notification.payload[@"dialogOk"] style:UIAlertActionStyleDefault
    handler:^(UIAlertAction *action) {
      [[AMMonitor sharedInstance] trackNotificationCallback:notification];
    }];
       [[AMMonitor sharedInstance] displayDialog:notification withOkAction:okAction];
};

- (BOOL)application:(UIApplication *)application
    didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
  // Override point for customization after application launch.
  [[AMMonitor sharedInstance] application:application didFinishLaunchingWithOptions:launchOptions];

    if (launchOptions) {
           [[AMMonitor sharedInstance] loadPayloadForNotification:launchOptions
                                                   andApplication:application
                                   loadCompletionHandlerWithError:^(AMNotification *notification, NSError *error) {
        if (error) {
        NSLog(@"Error occured while downloading notification :  %@", [error localizedDescription]);
        return;
        }
         // implement your own logic or use default
        [[AMMonitor sharedInstance] handleNotification:notification
                                          notificationHandler:nil
                                                dialogHandler:dialogHandler
                                                   urlHandler:nil
                                                 inAppHandler:nil];
                                   }];
        }
        
        //only for iOS > 8 (implentation for previous version omitted)
        UIUserNotificationType types = UIUserNotificationTypeBadge |
                UIUserNotificationTypeSound | UIUserNotificationTypeAlert;
        UIUserNotificationSettings *mySettings =
                [UIUserNotificationSettings settingsForTypes:types categories:nil];
        [[UIApplication sharedApplication] registerUserNotificationSettings:mySettings];
        [[UIApplication sharedApplication] registerForRemoteNotifications];
        
  return YES;
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult result))completionHandler {
    BOOL amPush = [[AMMonitor sharedInstance] loadPayloadForNotification:userInfo
                                                         andApplication:application loadCompletionHandlerWithError:^(AMNotification *notification, NSError *error) {
     if (error) {
      NSLog(@"Error occured while downloading notification :  %@", [error localizedDescription]);
    completionHandler(UIBackgroundFetchResultNoData);
    return;
    } else {
    completionHandler(UIBackgroundFetchResultNewData);
    }
    // implement your own logic or use default
    [[AMMonitor sharedInstance] handleNotification:notification notificationHandler:nil dialogHandler:dialogHandler urlHandler:nil inAppHandler:nil]; }];
    
    
    if (!amPush) {
       // handle not appmanago notifications here
       completionHandler(UIBackgroundFetchResultNewData);
    }
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
       [[AMMonitor sharedInstance] application:application didRegisterForRemoteNotificationsWithDeviceToken:deviceToken];
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
       [[AMMonitor sharedInstance] applicationDidEnterBackground:application];
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
       [[AMMonitor sharedInstance] applicationWillEnterForeground:application];
}

@end
