import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'am_monitor_plugin_platform_interface.dart';

/// An implementation of [AmMonitorPluginPlatform] that uses method channels.
class AmMonitorPlugin extends AmMonitorPluginPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('com.salesmanago/ammonitor');

  Future<void> create(String moduleSimpleId) async {
    await methodChannel
        .invokeMethod('create', {"moduleSimpleId": moduleSimpleId});
  }

  Future<void> destroy(String moduleSimpleId) async {
    await methodChannel
        .invokeMethod('destroy', {"moduleSimpleId": moduleSimpleId});
  }

  Future<void> eventStarted(String moduleSimpleId) async {
    await methodChannel
        .invokeMethod('eventStarted', {"moduleSimpleId": moduleSimpleId});
  }

  Future<void> eventEnded(String moduleSimpleId) async {
    await methodChannel
        .invokeMethod('eventEnded', {"moduleSimpleId": moduleSimpleId});
  }

  Future<void> clicked(String moduleSimpleId, String functionSimpleId) async {
    await methodChannel.invokeMethod('clicked', {
      "moduleSimpleId": moduleSimpleId,
      "functionSimpleId": functionSimpleId
    });
  }

  Future<void> eventCustom(String customEventName, [dynamic properties]) async {
    await methodChannel.invokeMethod('eventCustom',
        {"customEventName": customEventName, "properties": properties});
  }

  Future<void> syncEmail(String email) async {
    await methodChannel.invokeMethod('syncEmail', {"email": email});
  }

  Future<void> syncMsisdn(String msisdn) async {
    await methodChannel.invokeMethod('syncMsisdn', {"msisdn": msisdn});
  }

  Future<void> sendUserProperties([dynamic properties]) async {
    await methodChannel.invokeMethod('sendUserProperties', {"properties": properties});
  }

  Future<void> sendLocation(String latitude, String longitude) async {
    await methodChannel.invokeMethod('sendLocation', {"latitude": latitude,
      "longitude": longitude});
  }
}
